unit Test3DUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, Math, TeeProcs,
  TeEngine, Chart, ExtCtrls, TeeComma, TeeSurfa, TeeOpenGL, StdCtrls;

type TSpectraHeader = packed Record

//Field Name	        Dimension and Type	Notes
NumberOfSpectra1	      : SmallInt;     //2 bytes (int16)	Used up to version 1.02 (included) of DS450 MT2-DAD, then unused
NumberOfWavelengths	      : SmallInt;     //2 bytes (int16)
StartingWavelength	      : SmallInt;     //2 bytes (int16)
LastWavelength	              : SmallInt;     //2 bytes (int16)
DeltaLambda                   : SmallInt;     //2 bytes (int16)
SpectrumSize                  : Integer;      //4 bytes (int32)
DeviceUnitName                : array [0..9] of char; //10 bytes (string)
CorrespondentSampleFileName   : array [0..19] of char; //20 bytes (string)
ChannelName                   : array [0..15] of char; //16 bytes (string)
FileIdName                    : array [0..7] of char; //	8 bytes (string)
VoltageDimensionString	      : array [0..5] of char; //6 bytes (string)
TransformationFactor	      : Single;      //4 bytes (float32)
NumberOfSpectra2	      : Integer;      //4 bytes (int32)	Used from version 1.03 of DS450 MT2-DAD, before unused
SpectraStartingTime	      : Integer;      //4 bytes (int32)
SpectraEndingTime	      : Integer;      //4 bytes (int32)
SpectraTimeStep	              : Integer;      //4 bytes (int32)
SpectraFileType	              : Integer;      //4 bytes (int32)	0 Snapshot file1 Spectra library2 Spectra index3 Spectra under peak4 3d file5 Spectra file
MaxAbsorbanceValue	      : Integer;      //4 bytes (int32)
MinAbsorbanceValue	      : Integer;      //4 bytes (int32)
NumberOfSpectraIndex	      : Integer;      //4 bytes (int32)	Used only when < Spectra file type> is set to 2 (spectra index)
PeakDetectionWavelength	      : Integer;      //4 bytes (int32)	Used only when < Spectra file type> is set to 2 (spectra index)
PeakTableForSpectraIndex      : array [0..9] of char; //	10 bytes (string)	Used only when < Spectra file type> is set to 2 (spectra index)
ParameterTableForSpectraIndex : array [0..9] of char; //	10 bytes (string)	Used only when < Spectra file type> is set to 2 (spectra index)
TimeShiftForSpectraIndex      : Integer;      //4 bytes (int32)	Used only when < Spectra file type> is set to 2 (spectra index)
NotUsed	                      : array [0..117] of char; //118 bytes

end;


Type TSpectraSTRUCTURE1 = packed Record
//Spectra header - 66 * <Number of spectra 1 (or 2)>
//Field Name	Dimension and Type	Notes
TimeOfSpectrum	          : Integer;      //4 bytes (int32)
NormalisationFactor	  : Integer;      //4 bytes (float32)
NormalisationConstant 	  : Integer;      //4 bytes (int32)
LambdaMax                 : SmallInt;     //2 bytes (int16)
NumberOfValues	          : SmallInt;     //2 bytes (int16)	Is equal to:(<endWavelength> -<startWavelength>) /<stepWavelength> + 1
StartWavelength	          : Integer;      //4 bytes (int32)
EndWavelength	          : Integer;      //4 bytes (int32)
StepWavelength            : Integer;      //4 bytes (int32)
NameOfSpectrum            : array [0..20] of char;      //21 bytes (string)
NotUsed1                  : Char;         //1 byte
MaximumAbsorbance         : Integer;      //4 bytes (int32)
MinimumAbsorbance	  : Integer;      //4 bytes (int32)
BaselineCorrectionFlag	  : byte;         // 1 byte (int8)
NotUsed2                  : byte;          //1 byte
NotUsed3                  : array [0..5] of byte; //6 bytes
end;


const

RawTable : Array [0..255] of SmallInt = (
//0	1	2	3	4	5	6	7	8	9	10	11	12	13	14	15
1,	2,	3,	4,	5,	6,	7,	8,	9,	10,	11,	12,	13,	14,	15,	16,
17,	18,	19,	20,	21,	22,	23,	24,	25,	26,	27,	28,	29,	30,	31,	32,
34,	36,	38,	40,	42,	44,	46,	48,	50,	52,	54,	56,	58,	60,	62,	64,
66,	68,	70,	72,	74,	76,	78,	80,	82,	84,	86,	88,	90,	92,	94,	96,
100,	104,	108,	112,	116,	120,	124,	128,	132,	136,	140,	144,	148,	152,	156,	160,
164,	168,	172,	176,	180,	184,	188,	192,	196,	200,	204,	208,	212,	216,	220,	224,
232,	240,	248,	256,	264,	272,	280,	288,	296,	304,	312,	320,	328,	336,	344,	352,
360,	368,	376,	384,	392,	400,	408,	416,	424,	432,	440,	448,	456,	464,	472,	480,
496,	512,	528,	544,	560,	576,	592,	608,	624,	640,	656,	672,	688,	704,	720,	736,
752,	768,	784,	800,	816,	832,	848,	864,	880,	896,	912,	928,	944,	960,	976,	992,
1024,	1056,	1088,	1120,	1152,	1184,	1216,	1248,	1280,	1312,	1344,	1376,	1408,	1440,	1472,	1504,
1536,	1568,	1600,	1632,	1664,	1696,	1728,	1760,	1792,	1824,	1856,	1888,	1920,	1952,	1984,	2016,
2080,	2144,	2208,	2272,	2336,	2400,	2464,	2528,	2592,	2656,	2720,	2784,	2848,	2912,	2976,	3040,
3104,	3168,	3232,	3296,	3360,	3424,	3488,	3552,	3616,	3680,	3744,	3808,	3872,	3936,	4000,	4064,
4192,	4320,	4448,	4576,	4704,	4832,	4960,	5088,	5216,	5344,	5472,	5600,	5728,	5856,	5984,	6112,
6240,	6368,	6496,	6624,	6752,	6880,	7008,	7136,	7264,	7392,	7520,	7648,	7776,	7904,	8032,	8160
);


type
  TShow3DForm = class(TForm)
    OpenDialog: TOpenDialog;
    TeeOpenGL1: TTeeOpenGL;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    Chart1: TChart;
    Series1: TSurfaceSeries;
    Chart2: TChart;
    Panel1: TPanel;
    ZoomTrackBar: TTrackBar;
    WidthTrackBar: TTrackBar;
    Label1: TLabel;
    Label2: TLabel;
    Panel2: TPanel;
    RotationTrackBar: TTrackBar;
    Label3: TLabel;
    Panel3: TPanel;
    ElevationTrackBar: TTrackBar;
    Label4: TLabel;
    TeeCommander1: TTeeCommander;
    Label5: TLabel;
    OpenGLCheckBox: TCheckBox;
    TeeCommander2: TTeeCommander;
    SurfaceSeries1: TContourSeries;
    Panel4: TPanel;
    Button1: TButton;
    Edit1: TEdit;
    procedure Load3D (FName:String);
    Function GetRawValue (Index:Word):Single;
    procedure ZoomTrackBarChange(Sender: TObject);
    procedure OpenGLCheckBoxClick(Sender: TObject);
    procedure ElevationTrackBarChange(Sender: TObject);
    procedure RotationTrackBarChange(Sender: TObject);
    procedure WidthTrackBarChange(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    { Private-Deklarationen }
    SpectraHeader : TSpectraHeader;
    SpectraSTRUCTURE1 : TSpectraSTRUCTURE1;
    RawStream : TMemoryStream;
  public
    { Public-Deklarationen }
  end;

var
  Show3DForm: TShow3DForm;

implementation

{$R *.dfm}

Function TShow3DForm.GetRawValue (Index:Word):Single;
var Mant,Expo : Integer;
begin
     try
     Mant:=Index and $3ffe; // kick off sign and thereby multiply Mantisse by  2
     Expo:=Index shr 14;
     Result:= (Mant * IntPower(8,Expo) * 1.192093)/1000;
     if (Index and 1) = 1 then Result:=-Result;
     except
           Result:=0;
     end;
end;


{ this is a trick necessary... }
type TMyList=class(TChartValueList)
     end;


procedure TShow3DForm.Load3D (FName:String);
var n,x,index,i,Level:Integer;
    RawData:Word;
    XV,YV,ZV : Single;
begin
     Chart1.Series[0].Clear;
     RawStream:=TMemoryStream.Create;
     RawStream.LoadFromFile(FName);
     RawStream.Position:=0;
     RawStream.ReadBuffer (SpectraHeader,SizeOf(SpectraHeader));
     i:=0;
     try
        Level:=StrToInt(Trim(Edit1.Text));
     except
        Level:=-10;
     end;
     if SpectraHeader.NumberOfSpectra1>0 then i:=SpectraHeader.NumberOfSpectra1;
     if SpectraHeader.NumberOfSpectra2>0 then i:=SpectraHeader.NumberOfSpectra2;
     ZV:=0;
     with Chart1 do begin
          ClipPoints := True;
 //         AutoRepaint := False;
          Axes.FastCalc := true;
          View3DOptions.Orthogonal:=False;
     end;
     with (Chart1.Series[0] as TSurfaceSeries) do begin
          NumZValues:=1;
          NumXValues:=1;
 //         Wireframe:=True;
          IrregularGrid:=True;
          SmoothPalette:=True;
          UseColorRange:=False;
          UsePalette:=True;
          PaletteStyle:=psStrong;
          ShowInLegend:=False;
          ColorEachPoint:=False;
          FastBrush:=True;

     end;
     with Chart2 do begin
 //         ClipPoints := False;
 //         AutoRepaint := False;
          Axes.FastCalc := true;

     end;
     with (Chart2.Series[0] as TContourSeries) do begin
          NumZValues:=1;
          NumXValues:=1;
          IrregularGrid:=True;
          UseColorRange:=False;
          UsePalette:=True;
          PaletteStyle:=psStrong;
          ShowInLegend:=False;
          ColorEachPoint:=False;
     end;

     ZoomTrackBar.Position:=70;
     WidthTrackBar.Position:=100;


     for n:=1 to i do begin
         RawStream.ReadBuffer (SpectraSTRUCTURE1,SizeOf(SpectraSTRUCTURE1));
         XV:=SpectraSTRUCTURE1.StartWavelength/10;
         ZV:=SpectraSTRUCTURE1.TimeOfSpectrum/60000;
         for x:=1 to SpectraSTRUCTURE1.NumberOfValues do begin
             RawStream.ReadBuffer (RawData,SizeOf(RawData));
             try
                YV:=GetRawValue(RawData);
             except
                 YV:=0;
             end;
             YV:=0.2;  // value for hang program !!!
             With (Chart1.Series[0] as TSurfaceSeries) do begin
                  index := TMyList(XValues).AddChartValue(XV);
                  TMyList(ZValues).InsertChartValue(index,ZV);
                  TMyList(YValues).InsertChartValue(index,YV);
             end;
             With (Chart2.Series[0] as TContourSeries) do begin
                  index := TMyList(XValues).AddChartValue(ZV);
                  TMyList(ZValues).InsertChartValue(index,XV);
                  TMyList(YValues).InsertChartValue(index,YV);
             end;

             XV:=XV+(SpectraSTRUCTURE1.StepWavelength/10);
         end;
     end;
     Chart1.Repaint;
     RawStream.Free;
end;

procedure TShow3DForm.ZoomTrackBarChange(Sender: TObject);
begin
     Chart1.View3DOptions.Zoom:=ZoomTrackBar.Position;
     Chart1.Repaint;
end;

procedure TShow3DForm.OpenGLCheckBoxClick(Sender: TObject);
begin
     TeeOpenGL1.Active:= OpenGLCheckBox.Checked;
end;

procedure TShow3DForm.ElevationTrackBarChange(Sender: TObject);
begin
     Chart1.View3DOptions.Elevation:=ElevationTrackBar.Position;
     Chart1.Repaint;
end;

procedure TShow3DForm.RotationTrackBarChange(Sender: TObject);
begin
     Chart1.View3DOptions.Rotation:=RotationTrackBar.Position;
     Chart1.Repaint;
end;

procedure TShow3DForm.WidthTrackBarChange(Sender: TObject);
begin
     Chart1.Chart3DPercent:=WidthTrackBar.Position;
     Chart1.Repaint;
end;

procedure TShow3DForm.Button1Click(Sender: TObject);
begin
     if OpenDialog.Execute then Load3D (OpenDialog.FileName);
end;

end.
